﻿Type=Class
Version=4.2
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
' File: browser.bas
' Module Type: WebSocket class
' Purpose: Show a line chart with two lines using Google Line Chart with data from DHT11 connected to an Arduino running B4R.
' Changed: See readme.txt
' Notes:
' Google Line Chart: https://developers.google.com/chart/interactive/docs/gallery/linechart
'
Sub Class_Globals
	Private ws As WebSocket
	' Selector Labels
	Private lblapptitle As JQueryElement		'Title shown in the table header
	Private lblappfooter As JQueryElement
	Private lbllastupdate As JQueryElement
	Private lblconnectioninfo As JQueryElement
	Private lblabout As JQueryElement			'Abouttext shown in the jQM About Dialog 
	Private inputrefreshinterval As JQueryElement
	Private refreshinterval As Int
End Sub

Public Sub Initialize
'
End Sub

Private Sub WebSocket_Connected (WebSocket1 As WebSocket)
	DateTime.DateFormat = "dd.MM.yyyy"
	Log("WebSocket_Connected :: Start")
	'
	' Get as first task the websocket resulting in the index.html (ww folder) webpage displayed 
	ws = WebSocket1
	'Log("ID:" & ws.Session.Id)
	Main.ConnectedBrowsers.Put(Me, "")
	'
	' Update app version and the datetoday labels in the Index.html - these are defined linke <div id="lbldatetoday"></div>
	lblapptitle.SetText(Main.CAPPTITLE)
	lblabout.SetText(Main.CAPPTITLE & " " & Main.CAPPDESCRIPTION & " " & Main.CAPPVERSION & " " & Main.CAPPCOPYRIGHT)
	lblappfooter.SetText("FOOTER")
	lbllastupdate.SetText($"Updating the chart, please wait for sensor data."$)
	'
	'Load_Chart
End Sub

Private Sub WebSocket_Disconnected
	Main.ConnectedBrowsers.Remove(Me)
End Sub

'The main sub to create and display the chart
Public Sub Load_Chart(Params As Map)
	Log($"${DateTime.time(DateTime.Now)}: Load Chart"$)	
	'Define the columns as list. Two columns are used: Temperature and Humidity
	Dim cols As List
	cols.Initialize
	'Define the rows holding the data
	Dim rows As List
	rows.Initialize
	'Create the linechartdata taken from the datafile located in the dirapps folder
	CreateLineChartData(cols, rows)
	'Set the chart options
	Dim options As Map
	options.Initialize
	CreateLineChartOptions(options)
	'Build the data json string which is used to create the chart
	Dim jg As JSONGenerator
	jg.Initialize(CreateMap("data": CreateMap ("cols": cols, "rows": rows), "options": options))
	'Log(jg.ToPrettyString(2))
	Dim jsonstring As String = $"${jg.ToString}"$
	'Draw the chart by calling the function drawCharts defined in the index.html script section
	ws.RunFunction("drawCharts", Array As String(jsonstring))
	'Update label showing the time of the last update and the interval
	refreshinterval = Params.Get("measureinterval")
	lbllastupdate.SetText($"Last update: ${DateTime.Time(DateTime.Now)}, Chart Refresh Interval: ${refreshinterval}s"$)
	'Do notforget to flush which will show the updated values
	ws.Flush
End Sub

'
' Parameter id
'
' Parameter Label
' 
' Parameter dataType
' In line, area, bar, column AND candlestick charts (AND combo charts containing only such series), control the Type of the major axis:
' For a discrete axis, set the data column Type To String.
' For a continuous axis, set the data column Type To one of: number, date, DateTime OR timeofday.
' Examples:
' CreateColumn("", "Year", "string")
' CreateColumn("", "Sales", "number")
'
Private Sub CreateColumn(id As String, Label As String, dataType As String) As Map
	Return CreateMap("id": id, "label": Label, "type": dataType)
End Sub

Private Sub CreateRow(Values As List) As Map
	Dim vr As List
	vr.Initialize
	For Each o As Object In Values
		vr.Add(CreateMap("v": "" & o & ""))
	Next
	Return CreateMap("c": vr)
End Sub

Private Sub CreateLineChartData(cols As List, rows As List)
	'Log("CreateLineChartData :: Start")
	'Add the columns. Note that these are also used for the legend as set by the chart options
	cols.Add(CreateColumn("1", "Time", "string"))
	cols.Add(CreateColumn("2", "Temperature", "number"))
	cols.Add(CreateColumn("3", "Humidity", "number"))
	Dim l As List = File.ReadList(File.DirApp, Main.chartdatafile)
	'Log("Data Entries to load = " & l.Size)
	For i = 0 To l.Size - 1
		Dim s() As String = Regex.Split(";", l.Get(i))
		If s.Length = 3 Then
			'Log(s(0) & "," & s(1) & "," & s(2))
			rows.Add(CreateRow(Array As Object(s(0), s(1), s(2))))
		End If
	Next
	'Log("CreateLineChartData :: End")
End Sub

'Set the linechart options. Showed also the javaScript declaration to learn how to convert to B4J
Private Sub CreateLineChartOptions (options As Map)
	'Chart options
	'NOT USED is the title
	'title: 'Temperature and Humidity Data',
	'options.Put("title", "Temperature and Humidity Data")
	'curveType: 'function'
	options.Put("curveType", "function")
	'legend: { position: 'bottom' }
	options.Put("legend", CreateMap("position":"bottom"))
	'chart area background color
	options.Put("chartArea", CreateMap("backgroundColor":"#fdc"))
	'Dual axes classic way with titles
	'IMPORTANT: Ensure to set the right case for the properties. See Google Line Chart Ref.
	'series: {0: {targetAxisIndex: 0},1: {targetAxisIndex: 1}},
	Dim mseries As Map
	mseries.Initialize
	mseries.Put("0": CreateMap("targetAxisIndex":0))
	mseries.Put("1": CreateMap("targetAxisIndex":1))
	options.Put("series", mseries)
	'vAxes: {0: {title: 'Temperature (Celsius)'},1: {title: 'Humidity (%RH)'}},
	Dim mvaxes As Map
	mvaxes.Initialize
	mvaxes.Put("0", CreateMap("title":"Temperature (Celsius)"))
	mvaxes.Put("1", CreateMap("title":"Humidity (%RH)"))
	options.Put("vAxes", mvaxes)
End Sub

#Region BUTTONS
'Handle button clicks as defined in index.html

'Handle click on the button with id btnAbout - Note in the index.html all id's are in lowercase
'A w3.css modal dialog is shown
Sub btnAbout_click(params As Map)
	Log(DateTime.Time(DateTime.Now) & " :: btnAbout clicked")
End Sub

'Handle the button OK click from the InputDialog RefreshInterval (via w3.css modal dialog).
'The inputnumber is checked, then send to the connected arduinoas a function.
Sub btnInputRefreshIntervalOk_click(params As Map)
	Log(DateTime.Time(DateTime.Now) & " :: btnInputRefreshIntervalOk - " & inputrefreshinterval.GetVal.Value)
	Dim ris As String = inputrefreshinterval.GetVal.Value
	If IsNumber(ris) Then
		'Check if refreshinterval not < 1s
		Dim ri As Int = ris
		If ri < 1 Then Return
		refreshinterval = ri
		'Publish the message to the client, which has to take action
		Log($"Updating refresh interval to ${refreshinterval}"$)
		CallSubDelayed2(Main.ConnectedArduino, "UpdateInterval", refreshinterval )
	End If
End Sub
#End Region
